import { scrollPosition } from "./scroll-delegate";
function listen(router, history) {
  if (import.meta.env.SSR)
    return;
  history.scrollRestoration = "manual";
  addEventListener("beforeunload", () => {
    history.scrollRestoration = "auto";
  });
  addEventListener("load", () => {
    history.scrollRestoration = "manual";
  });
  const prefetched = /* @__PURE__ */ new Set();
  const triggerPrefetch = (event) => {
    const a = findAnchor(event.target);
    if (a && a.href && a.hasAttribute("data-prefetch") && !prefetched.has(a.href)) {
      router.prefetch(getHref(a));
      prefetched.add(a.href);
    }
  };
  let mouseMoveTimeout;
  const handleMouseMove = (event) => {
    clearTimeout(mouseMoveTimeout);
    mouseMoveTimeout = setTimeout(() => {
      triggerPrefetch(event);
    }, 30);
  };
  addEventListener("touchstart", triggerPrefetch);
  addEventListener("mousemove", handleMouseMove);
  let hashNavigation = false;
  addEventListener("click", (event) => {
    if (router.disabled)
      return;
    if (event.button || event.which !== 1)
      return;
    if (event.metaKey || event.ctrlKey || event.shiftKey || event.altKey)
      return;
    if (event.defaultPrevented)
      return;
    const a = findAnchor(event.target);
    if (!a || !a.href)
      return;
    const url = getHref(a);
    const isSvgAEl = a instanceof SVGAElement;
    if (!isSvgAEl && url.origin === "null")
      return;
    const rel = (a.getAttribute("rel") || "").split(/\s+/);
    if (a.hasAttribute("download") || rel.includes("external"))
      return;
    if (isSvgAEl ? a.target.baseVal : a.target)
      return;
    const [base, hash] = url.href.split("#");
    if (hash !== void 0 && base === location.href.split("#")[0]) {
      hashNavigation = true;
      router.hashChanged(hash);
      router.scroll.savePosition?.();
      return;
    }
    router._navigate(url, {
      scroll: !a.hasAttribute("data-noscroll") ? () => scrollPosition() : null,
      replace: url.href === location.href,
      keepfocus: false,
      accepted: () => event.preventDefault(),
      blocked: () => event.preventDefault()
    });
  });
  addEventListener("visibilitychange", () => {
    if (document.visibilityState === "hidden") {
      router.scroll.savePosition?.();
      router.scroll.commit?.();
    }
  });
  addEventListener("popstate", (event) => {
    if (!event.state || router.disabled)
      return;
    if (event.state[router._historyKey] === router._historyIndex)
      return;
    router._navigate(new URL(location.href), {
      scroll: () => router.scroll.getSavedPosition?.(event.state),
      keepfocus: false,
      accepted: () => {
        router._historyIndex = event.state[router._historyKey];
      },
      blocked: () => {
        const delta = router._historyIndex - event.state[router._historyKey];
        history.go(delta);
      }
    });
  });
  addEventListener("hashchange", () => {
    if (hashNavigation) {
      hashNavigation = false;
      history.replaceState(
        {
          ...history.state,
          [router._historyKey]: ++router._historyIndex
        },
        "",
        location.href
      );
    }
  });
}
function getHref(node) {
  return node instanceof SVGAElement ? new URL(node.href.baseVal, document.baseURI) : new URL(node.href);
}
function findAnchor(node) {
  while (node && node.nodeName.toUpperCase() !== "A")
    node = node.parentNode;
  return node;
}
export {
  listen
};
