import { isString, isUndefined } from "../../shared";
const SCROLL_KEY = "vitebook:scroll";
const createSimpleScrollDelegate = (router) => {
  let scrollPositions = {};
  if (!import.meta.env.SSR) {
    try {
      scrollPositions = JSON.parse(sessionStorage[SCROLL_KEY]);
    } catch {
    }
    const scroll = scrollPositions[router._historyIndex];
    if (scroll) {
      history.scrollRestoration = "manual";
      window.scrollTo(scroll.left, scroll.top);
    }
  }
  return {
    savePosition() {
      if (import.meta.env.SSR)
        return;
      scrollPositions[router._historyIndex] = scrollPosition();
    },
    getSavedPosition(state) {
      return scrollPositions[state[SCROLL_KEY]];
    },
    commit() {
      if (import.meta.env.SSR)
        return;
      try {
        sessionStorage[SCROLL_KEY] = JSON.stringify(scrollPositions);
      } catch {
      }
    }
  };
};
const createComplexScrollDelegate = (router) => {
  let scrollPositions = {}, _base, _behavior;
  if (!import.meta.env.SSR) {
    try {
      scrollPositions = JSON.parse(sessionStorage[SCROLL_KEY]);
    } catch {
    }
    const scroll2 = scrollPositions[router._historyIndex];
    if (scroll2) {
      history.scrollRestoration = "manual";
      window.scrollTo(scroll2.left, scroll2.top);
    }
  }
  async function scroll({ target, hash, from, to }) {
    if (import.meta.env.SSR)
      return;
    let cancelled = false;
    const cancel = () => {
      cancelled = true;
    };
    let scrollTarget = null;
    if (target) {
      scrollTarget = await target({ cancel });
    } else if (!isUndefined(from) && to && _behavior) {
      scrollTarget = await _behavior({
        from,
        to,
        cancel,
        savedPosition: scrollPositions[router._historyIndex]
      });
    }
    if (scrollTarget === false) {
      cancel();
      return;
    }
    if (cancelled)
      return;
    const deepLinked = hash ? document.getElementById(decodeURI(hash).slice(1)) : null;
    const scrollTo = (options) => {
      if ("scrollBehavior" in document.documentElement.style) {
        window.scrollTo(options);
      } else {
        window.scrollTo(options.left ?? 0, options.top ?? 0);
      }
    };
    const base = _base?.();
    const baseTop = base?.top ?? 0;
    const baseLeft = base?.left ?? 0;
    const baseBehavior = base?.behavior ?? "auto";
    if (scrollTarget || deepLinked) {
      const el = isString(scrollTarget?.el) ? document.querySelector(scrollTarget.el) : scrollTarget?.el ?? deepLinked;
      const docRect = document.documentElement.getBoundingClientRect();
      const elRect = el?.getBoundingClientRect() ?? { top: 0, left: 0 };
      const offsetTop = baseTop + (scrollTarget?.top ?? 0);
      const offsetLeft = baseLeft + (scrollTarget?.left ?? 0);
      const behavior = scrollTarget?.behavior ?? baseBehavior;
      scrollTo({
        left: elRect.left - docRect.left - offsetLeft,
        top: elRect.top - docRect.top - offsetTop,
        behavior
      });
    } else {
      scrollTo({ top: baseTop, left: baseLeft });
    }
  }
  const manager = {
    scroll,
    setBase(base) {
      _base = base;
    },
    setBehavior(behavior) {
      _behavior = behavior;
    },
    savePosition() {
      if (import.meta.env.SSR)
        return;
      scrollPositions[router._historyIndex] = scrollPosition();
    },
    getSavedPosition(state) {
      return scrollPositions[state[SCROLL_KEY]];
    },
    commit() {
      if (import.meta.env.SSR)
        return;
      try {
        sessionStorage[SCROLL_KEY] = JSON.stringify(scrollPositions);
      } catch {
      }
    }
  };
  return manager;
};
function scrollPosition() {
  return {
    top: scrollY,
    left: scrollX
  };
}
export {
  createComplexScrollDelegate,
  createSimpleScrollDelegate,
  scrollPosition
};
